CON

  'Simple program to write the contents of the DAT section to the upper section of EEPROM.
  'It also prints out the addresses of the loctions that it is writing for later reference.

  {
  TODO:
        Create a new read routine that takes a count argument and reads that many bytes via the ReadPage command.

        Convert "readString to a generic ReadBlock routine.
  '}  


  ' For some reason the first page in the upper section of eeprom gets written by the Propeller Tool
  ' upon program write. So we must start on the next page.

  eepromBase   = $8000 + EEPROMPageSize
  
  EEPROMPageSize        = 128  'For the AT512
  MAXAddress            = $FFFF

  Normal        = 0
  Partial       = 1

OBJ

  I2C      : "Basic_I2C_Driver.spin"

VAR

  byte  PageBuffer[EEPROMPageSize]
  long  bufferLength
  long  EEPROMAddress

PUB init | temp

  'Set up I2C Driver
  I2C.Initialize(i2c#BootPin)

  EEPROMAddress :=  eepromBase
  bytefill(@PageBuffer, 0, EEpromPageSize)

PUB getBlockSize(address) : count

  count := I2C.ReadWord(i2c#BootPin, i2c#EEPROM, address) 

PUB readBlock(address, _byteArray) : error | ToAddress, FromAddress, count, value

  error := false
  
  FromAddress := address
  ToAddress := _byteArray
  count := I2C.ReadWord(i2c#BootPin, i2c#EEPROM, FromAddress)

  FromAddress += 2
  repeat while count > 0
    value := I2C.ReadPage(i2c#BootPin, i2c#EEPROM, FromAddress, @PageBuffer, count <# EEPROMPageSize)
    if value == true
      error := true
      quit
    else
      bytemove(ToAddress, @PageBuffer, count <# EEPROMPageSize) 
      count -= EEPROMPageSize
      FromAddress += EEPROMPageSize
      ToAddress += EEPROMPageSize
   

PUB writeString(address) : returnAddress| StringPointer, temp, count

  returnAddress := EEPROMAddress + bufferlength

  StringPointer := address
  count := strSize(StringPointer) + 1

  byte[@PageBuffer][bufferlength] := (count) & $FF
  bufferlength++
  if EEpromPageSize-bufferLength == 0
    DumpPage(Normal)
  byte[@PageBuffer][bufferlength] := ((count) >> 8) & $FF 
  bufferlength++
    
  repeat while count > 0
    temp := EEpromPageSize-bufferlength <# count
    bytemove(@PageBuffer+bufferlength, Stringpointer, temp)
    StringPointer += temp
    count -= temp
    bufferlength += temp          

    if DumpPage(Normal)
      quit

  return
  
PUB WriteBlock(address, numBytes) : returnAddress | dataPointer, temp, count

  returnAddress := EEPROMAddress + bufferlength
  dataPointer := address
  count := numBytes

  byte[@PageBuffer][bufferlength] := (numBytes) & $FF
  bufferlength++
  if EEpromPageSize-bufferLength == 0
    DumpPage(Normal)
  byte[@PageBuffer][bufferlength] := ((numBytes) >> 8) & $FF 
  bufferlength++
  
  repeat while count > 0
    temp := EEpromPageSize-bufferlength <# count

    bytemove(@PageBuffer+bufferlength, dataPointer, temp)
    dataPointer += temp
    count -= temp
    bufferlength += temp          

    if DumpPage(Normal)
      quit

  return

PUB finishWriting : error

  error := DumpPage(Partial)

PUB setAddress(value)

  if value => $8000 AND value =< $FFFF
    EEPROMAddress := value

  return
  
PRI DumpPage(mode) : error | temp, count

  repeat while I2C.WriteWait(i2c#BootPin, i2c#EEPROM, EEPROMAddress)

  error := false

  if mode == Partial
    count := EEpromPageSize
  else
    count := bufferlength

  if count > (MAXAddress - EEPROMAddress) + 1
    'fds.str(DEBUG, String("   Memory Full.", 13))
    error := true
    return
       
  if count == EEpromPageSize  
    temp := I2C.WritePage(i2c#BootPin, i2c#EEPROM, EEPROMAddress, @PageBuffer, EEpromPageSize)
    if temp == 0

      EEPROMAddress += EEPROMPageSize
      bufferlength := 0
      bytefill(@PageBuffer, 0, EEpromPageSize)
    else
      error := true
  else
    error := true